<?php
namespace plugins\polls\controllers;

use plugins\polls\services\Vote;
use plugins\polls\services\Clear;

/**
 * Vote controller
 * @author Jérôme Brilland
 * @version 1.0
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
 class VoteController extends \ActionController {
          
     protected function initialize() {
         $i18n = \IoC::resolve( 'i18n' );
         
         $eventMediator = \IoC::resolve( 'eventmediator' );
         
         $eventMediator->attach( 'invalid_poll', function() use( $i18n ) {
             $this->flash['error'] = $i18n->__( 'polls.invalid_poll' );
             $this->forward( 'controllers\\ErrorController', '400' );
         } );
         $eventMediator->attach( 'poll_closed', function() use( $i18n ) {
             $this->flash['error'] = $i18n->__( 'polls.poll_closed' );
             $this->forward( 'controllers\\ErrorController', '400' );
         } );
         $eventMediator->attach( 'already_voted', function( $pollId ) use( $i18n ) {
             $this->flash['error'] = $i18n->__( 'polls.already_voted' );
             $this->redirect( BASE_URL, 400 );
         } );
     }
     
     public function doAdd() {    
         if ( $this->request->hasParameter( 'voteoption' ) ) {
             $vote = new Vote( $this->request->getAttribute( 'id' ), $this->request->getParameters()->getInt( 'voteoption') );
         } else {
             $vote = new Vote( $this->request->getAttribute( 'id' ), array_filter( $this->request->getParameters()->getArray( 'voteoptions' ), 'ctype_digit' ) );
         }   
         $vote->perform();
         $i18n = \IoC::resolve( 'i18n' );
         $this->flash['success'] = $i18n->__( 'polls.thankyou' );
         $this->redirect( BASE_URL );
     }

     public function doClear() {
       $authorization = \IoC::resolve( 'authorization' );
       $authorization->authorize( 'polls.update_poll' );
       $clear = new Clear( $this->request->getAttribute( 'id' ) );
       $clear->perform();
       $this->redirect( BASE_URL . 'polls/poll/show/' . $this->request->getAttribute( 'id' ) );
     }
 }
