<?php
if ( !defined( 'IN_CMS' ) ) {
    exit();
}

use models\Template;
use models\Role;
use models\Permission;
use models\RolePermission;

$schema = \IoC::resolve( 'sqlschema' );

$schema->dropTableIfexists( 'vote_track' );
$schema->dropTableIfexists( 'vote_option' );
$schema->dropTableIfexists( 'poll' );

$schema->createTable( 'poll', [
    'poll_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'topic' => ['text', ['not_null' => true]],
    'language' => ['varchar', ['length' => 2, 'default'=> 'NULL']],
    'user_id' => ['integer', ['default' => 'NULL']],
    'allow_multiple_choice' => ['boolean', ['not_null' => true]],
    'start_date' => ['date', ['not_null' => true]],
    'start_time' => ['time', ['not_null' => true]],
    'end_date' => ['date', ['default' => 'NULL']],
    'end_time' => ['time', ['default' => 'NULL']],
    'created_at' => ['datetime', ['not_null' => true]],
    'updated_at' => ['datetime', ['default' => 'NULL']]
] );

$schema->addForeignKey( 'FK_poll', 'poll', 'user_id', 'user', 'user_id', 'SET NULL', 'CASCADE' );

$schema->createTable( 'vote_option', [
    'vote_option_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'title' => ['varchar', ['length' => 255, 'not_null' => true]],
    'poll_id' => ['integer', ['default' => 'NULL']],
    'votes' => ['integer', ['not_null' => true]],
    'created_at' => ['datetime', ['not_null' => true]],
    'updated_at' => ['datetime', ['default' => 'NULL']]
] );

$schema->addForeignKey( 'FK_vote_option', 'vote_option', 'poll_id', 'poll', 'poll_id', 'CASCADE', 'CASCADE' );

$schema->createTable( 'vote_track', [
    'vote_track_id' => ['integer', ['not_null' => true, 'primary_key' => true, 'identity' => true]],
    'poll_id' => ['integer', ['not_null' => true]],
    'hashed_fingerprint' => ['varchar', ['length' => 50, 'not_null' => true]],
    'created_at' => ['datetime', ['not_null' => true]]
] );

$schema->addForeignKey( 'FK_vote_track', 'vote_track', 'poll_id', 'poll', 'poll_id', 'CASCADE', 'CASCADE' );

$configurationHandler = \IoC::resolve( 'databaseConfigurationHandler' );
$settings = new \Configuration( $configurationHandler );

$template = new Template();
$template->name = 'Poll';
$template->content_type = 'text/html';
$template->content = file_get_contents ( dirname( __FILE__ ) . DS . TEMPLATES_DIR . 'poll' . DS . 'poll.html' );
$template->save();

$settings['polls.poll_template'] = $template->template_id;

$configurationHandler->save();

$permissions = [
    'polls.view_polls',
    'polls.create_poll',
    'polls.update_poll',
    'polls.delete_poll'
];

$administrator = Role::findOne( ['name' => 'administrator'] );
$editor = Role::findOne( ['name' => 'editor'] );

foreach ( $permissions as $name ) {
    $permission = new Permission( ['name' => $name] );
    $permission->save();
            
    $rolePermission = new RolePermission( ['role_id' => $administrator->role_id, 'permission_id' => $permission->permission_id] );
    $rolePermission->save();

    $rolePermission = new RolePermission( ['role_id' => $editor->role_id, 'permission_id' => $permission->permission_id] );
    $rolePermission->save();
}

$logger = \IoC::resolve( 'logger' );
$logger->info( 'Plugin polls successfully installed.' );

return true;
