<?php
namespace plugins\polls\services;

use plugins\polls\models\Poll;
use plugins\polls\models\VoteTrack;
use plugins\polls\models\VoteOption;

/**
 * @author Jérôme Brilland
 * @version 1.0
 * @copyright (c) 2022, Jérôme Brilland
 * @license http://www.gnu.org/licenses/gpl.txt GNU General Public License
 */
class Vote {
    
    private $pollId;
    private $voteOptions;
    
    /**
     * @param int $pollId Poll id
     * @param int|array $voteOptions
     */
    public function __construct( int $pollId, $voteOptions ) {
        $this->pollId = $pollId;
        $this->voteOptions = $voteOptions;
    }
    
    public function perform() {
        $eventMediator = \IoC::resolve( 'eventmediator' );
        
        $poll = Poll::findOne( $this->pollId );
        if ( empty( $poll ) ) {
            $eventMediator->trigger( 'invalid_poll' );
            return;
        } else {
            if ( !$poll->isOpen() ) {
                $eventMediator->trigger( 'poll_closed' );
                return;
            }
        }
        
        $fingerprint = hash( 'ripemd160', $_SERVER['REMOTE_ADDR'] . $_SERVER['HTTP_USER_AGENT'] . $_SERVER['HTTP_ACCEPT_LANGUAGE'] );
        if ( VoteTrack::exists( ['poll_id' => $this->pollId, 'hashed_fingerprint' => $fingerprint ] ) ) {
            $eventMediator->trigger( 'already_voted', [$this->pollId] );
            return;
        }
        $database = \IoC::resolve( 'database' );
        $database->action( function( $database ) use( $fingerprint, $poll ) {
            if ( $poll->allow_multiple_choice ) {
                foreach( $this->voteOptions as $voteOptionId => $value ) {
                    if ( (boolean)$value ) {
                        $voteOption = VoteOption::findOne( '*', ['poll_id' => $this->pollId, 'vote_option_id' => (int)$voteOptionId] );
                        if ( empty( $voteOption ) ) {
                            return false;
                        }
                        $voteOption->votes += 1;
                        $voteOption->save();
                    }
                }
            } else {
                $voteOption = VoteOption::findOne( '*', ['poll_id' => $this->pollId, 'vote_option_id' => (int)$this->voteOptions] );
                if ( empty( $voteOption ) ) {
                    return false;
                }
                $voteOption->votes += 1;
                $voteOption->save();
            }
            
            $voteTrack = new VoteTrack();
            $voteTrack->poll_id = $this->pollId;
            $voteTrack->hashed_fingerprint = $fingerprint;
            $voteTrack->save();
        } );
        $eventMediator->trigger( 'vote_recorded', [$database->id()] );
    }
}
